@@ -4,6 +4,38 @@
 ;; Done by fairly faithful modification of:
 ;; c-mode.el, Copyright (C) 1985 Richard M. Stallman.
 ;;
+;; Oct, 1989 (Dave Detlefs, dld@cs.cmu.edu)
+;;   Added contribution from Igor Metz <metz@iam.unibe.ch>:
+;;     functions c++-comment-region and c++-uncomment-region and 
+;;     corresponding key-binding.
+;;   Also fixed bug in indentation of second line after an empty
+;;   arglist with empty-arglist non-null.
+;;   
+;; Sept, 1989 (Glen Ditchfield, gjditchfield@violet.uwaterloo.ca):
+;;   Textual changes to more closely imitate Emacs 18.55's c-mode.
+;;   Fixed handling of "default:", where ":" was the last character in the
+;;   buffer.  Fixed indentation of comments starting in column 0, and when
+;;   previous line contained more than one comment start string.  Fixed
+;;   handling of "friend class".
+;;
+;; Aug 7, 1989; John Hagerman (hagerman@ece.cmu.edu):
+;;   Changed calculate-c++-indent to handle member initializations
+;;   more flexibly. Two new variables are used to control behavior:
+;;   c++-member-init-indent and c++-continued-member-init-offset.
+;;   Note the assumption that member initializations and argument
+;;   declarations are not mixed in one function definition.
+;;
+;; June 1989 (Dave Detlefs, dld@cs.cmu.edu)
+;;   Fixed calculate-c++-indent to handle continued lines ending in
+;;   {'s.  (I wasn't following C-mode closely enough, or C-mode
+;;   changed.)  Made ' a quote character, at the behest of someone
+;;   whose mail I apparently deleted (if they send me mail I'll credit
+;;   them here in a future revision.)
+;;   Dan Weinreb (dlw@odi.com) pointed out that 'c++-mode successively
+;;   bound c++-indent-exp and c++-indent-defun to ESC-^q.  ESC-^q is
+;;   now bound to c++-indent-exp, while, c++-indent-defun is invoked
+;;   with ESC-^x.
+
 ;; February 1989 (Dave Detlefs, dld@cs.cmu.edu)
 ;;   Fixed some errors in c++-indent-defun, as pointed out by Sam
 ;;   Haradhvala (odi!sam@talcott.harvard.edu).
@@ -29,28 +61,46 @@
 ;;
 ;; August 1987: incorporated changes done by Glen Ditchfield of Waterloo.
 
-(defvar c++-mode-syntax-table nil
-  "Syntax table in use in C++-mode buffers.")
 (defvar c++-mode-abbrev-table nil
   "Abbrev table in use in C++-mode buffers.")
+(define-abbrev-table 'c++-mode-abbrev-table ())
 
-(defvar c++-mode-map (make-sparse-keymap)
+(defvar c++-mode-map ()
   "Keymap used in C++ mode.")
+(if c++-mode-map
+    ()
+  (setq c++-mode-map (make-sparse-keymap))
+  (define-key c++-mode-map "\C-j" 'reindent-then-newline-and-indent)
+  (define-key c++-mode-map "{" 'electric-c++-brace)
+  (define-key c++-mode-map "}" 'electric-c++-brace)
+  (define-key c++-mode-map ";" 'electric-c++-semi)
+  (define-key c++-mode-map "\e\C-h" 'mark-c-function)
+  (define-key c++-mode-map "\e\C-q" 'indent-c++-exp)
+  (define-key c++-mode-map "\177" 'backward-delete-char-untabify)
+  (define-key c++-mode-map "\t" 'c++-indent-command)
+  (define-key c++-mode-map "\C-c\C-i" 'c++-insert-header)
+  (define-key c++-mode-map "\C-c\C-\\" 'c++-macroize-region)
+  (define-key c++-mode-map "\C-c\C-c" 'c++-comment-region)
+  (define-key c++-mode-map "\C-c\C-u" 'c++-uncomment-region)
+  (define-key c++-mode-map "\e\C-a" 'c++-beginning-of-defun)
+  (define-key c++-mode-map "\e\C-e" 'c++-end-of-defun)
+  (define-key c++-mode-map "\e\C-x" 'c++-indent-defun))
 
-(define-key c++-mode-map "\C-j" 'reindent-then-newline-and-indent)
-(define-key c++-mode-map "{" 'electric-c++-brace)
-(define-key c++-mode-map "}" 'electric-c++-brace)
-(define-key c++-mode-map ";" 'electric-c++-semi)
-(define-key c++-mode-map "\e\C-h" 'mark-c-function)
-(define-key c++-mode-map "\e\C-q" 'indent-c++-exp)
-(define-key c++-mode-map "\177" 'backward-delete-char-untabify)
-(define-key c++-mode-map "\t" 'c++-indent-command)
-(define-key c++-mode-map "\C-c\C-i" 'c++-insert-header)
-(define-key c++-mode-map "\C-c\C-\\" 'c++-macroize-region)
-(define-key c++-mode-map "\e\C-a" 'c++-beginning-of-defun)
-(define-key c++-mode-map "\e\C-e" 'c++-end-of-defun)
-(define-key c++-mode-map "\e\C-q" 'c++-indent-defun)
+(defvar c++-mode-syntax-table nil
+  "Syntax table in use in C++-mode buffers.")
 
+(if c++-mode-syntax-table
+    ()
+  (setq c++-mode-syntax-table (copy-syntax-table c-mode-syntax-table))
+  (modify-syntax-entry ?/ ". 12" c++-mode-syntax-table)
+  (modify-syntax-entry ?\n ">" c++-mode-syntax-table)
+  (modify-syntax-entry ?\' "." c++-mode-syntax-table))
+
+(defvar c++-continued-member-init-offset nil
+  "*Extra indent for continuation lines of member inits; NIL means to align
+with previous initializations rather than with the colon on the first line.")
+(defvar c++-member-init-indent 0
+  "*Indentation level of member initializations in function declarations.")
 (defvar c++-friend-offset -4
   "*Offset of C++ friend class declarations relative to member declarations.")
 (defvar c++-electric-colon t
@@ -59,7 +109,6 @@
   "*Indicates how far to indent an line following an empty argument
 list.  Nil indicates to just after the paren.")
 
-
 
 (defun c++-mode ()
   "Major mode for editing C++ code.  Very much like editing C code.
@@ -84,6 +133,9 @@ Variables controlling indentation style:
  c-continued-statement-offset
     Extra indentation given to a substatement, such as the
     then-clause of an if or body of a while.
+ c-continued-brace-offset
+    Extra indentation given to a brace that starts a substatement.
+    This is in addition to c-continued-statement-offset.
  c-brace-offset
     Extra indentation for line if it starts with an open brace.
  c-brace-imaginary-offset
@@ -92,7 +144,8 @@ Variables controlling indentation style:
  c-argdecl-indent
     Indentation level of declarations of C function arguments.
  c-label-offset
-    Extra indentation for line that is a label, or case or default.
+    Extra indentation for line that is a label, or case or ``default:'', or
+    ``public:'' or ``private:'', or ``protected:''.
  c++-electric-colon
     If non-nil at invocation of c++-mode (t is the default) colon electricly
     indents.
@@ -102,6 +155,23 @@ Variables controlling indentation style:
     left paren.
  c++-friend-offset
     Offset of C++ friend class declarations relative to member declarations.
+ c++-member-init-indent
+    Indentation level of member initializations in function declarations,
+    if they are on a separate line beginning with a colon.
+ c++-continued-member-init-offset
+    Extra indentation for continuation lines of member initializations; NIL
+    means to align with previous initializations rather than with the colon.
+
+Settings for K&R, BSD, and Stroustrup indentation styles are
+  c-indent-level                5    8    4
+  c-continued-statement-offset  5    8    4
+  c-continued-brace-offset                0
+  c-brace-offset               -5   -8    0
+  c-brace-imaginary-offset                0
+  c-argdecl-indent              0    8    4
+  c-label-offset               -5   -8   -4
+  c++-empty-arglist-indent                4
+  c++-friend-offset                       0
 
 Turning on C++ mode calls the value of the variable c++-mode-hook with
 no args,if that value is non-nil."
@@ -110,26 +180,14 @@ no args,if that value is non-nil."
   (use-local-map c++-mode-map)
   (setq major-mode 'c++-mode)
   (setq mode-name "C++")
-  (define-abbrev-table 'c++-mode-abbrev-table ())
   (setq local-abbrev-table c++-mode-abbrev-table)
-  (if (not c++-mode-syntax-table)
-      (let ((i 0))
-	(setq c++-mode-syntax-table (make-syntax-table))
-	(set-syntax-table c++-mode-syntax-table)
-	(modify-syntax-entry ?\\ "\\")
-	(modify-syntax-entry ?/ ". 12")
-	(modify-syntax-entry ?\n ">")
-	(modify-syntax-entry ?+ ".")
-	(modify-syntax-entry ?- ".")
-	(modify-syntax-entry ?= ".")
-	(modify-syntax-entry ?% ".")
-	(modify-syntax-entry ?< ".")
-	(modify-syntax-entry ?> "."))
-      (set-syntax-table c++-mode-syntax-table))
+  (set-syntax-table c++-mode-syntax-table)
   (make-local-variable 'paragraph-start)
   (setq paragraph-start (concat "^$\\|" page-delimiter))
   (make-local-variable 'paragraph-separate)
   (setq paragraph-separate paragraph-start)
+  (make-local-variable 'paragraph-ignore-fill-prefix)
+  (setq paragraph-ignore-fill-prefix t)
   (make-local-variable 'indent-line-function)
   (setq indent-line-function 'c++-indent-line)
   (make-local-variable 'require-final-newline)
@@ -150,24 +208,25 @@ no args,if that value is non-nil."
   (if c++-electric-colon
       (define-key c++-mode-map ":" 'electric-c++-terminator)))
 
-
 ;; This is used by indent-for-comment
 ;; to decide how much to indent a comment in C++ code
 ;; based on its context.
-
 (defun c++-comment-indent ()
-  (let (cur-col)
+  (if (looking-at "^\\(/\\*\\|//\\)")
+      0					;Existing comment at bol stays there.
     (save-excursion
       (skip-chars-backward " \t")
-      (setq cur-col (current-column))
-      (end-of-line 0)
-      (if (re-search-backward comment-start-skip
-			      (save-excursion (beginning-of-line 1) (point))
-			      t)
-	  (if (= (current-column) 0) 0
-	    (max (1+ cur-col) (current-column)))
-	; Else indent at comment column, except leave at least one space.
-	(max (1+ cur-col) comment-column)))))
+      (max
+       ;; leave at least one space on non-empty lines.
+       (if (zerop (current-column)) 0 (1+ (current-column)))
+       (let ((cur-pt (point)))
+	 (beginning-of-line 0)
+	 ;; If previous line had a comment, use it's indent
+	 (if (re-search-forward comment-start-skip cur-pt t)
+	     (progn
+	       (goto-char (match-beginning 0))
+	       (current-column))
+	   comment-column))))))		;otherwise indent at comment column.
 
 (defun electric-c++-brace (arg)
   "Insert character and correct line's indentation."
@@ -178,16 +237,15 @@ no args,if that value is non-nil."
 	     (or (save-excursion
 		   (skip-chars-backward " \t")
 		   (bolp))
-		 (if c-auto-newline
-		     (progn (c++-indent-line) (newline) t)
-		   nil)))
+		 (if c-auto-newline (progn (c++-indent-line) (newline) t) nil)))
 	(progn
 	  (insert last-command-char)
 	  (c++-indent-line)
 	  (if c-auto-newline
 	      (progn
-		(setq insertpos (1- (point)))
 		(newline)
+		;; (newline) may have done auto-fill
+		(setq insertpos (- (point) 2))
 		(c++-indent-line)))
 	  (save-excursion
 	    (if insertpos (goto-char (1+ insertpos)))
@@ -214,17 +272,32 @@ no args,if that value is non-nil."
 		    (beginning-of-line)
 		    (skip-chars-forward " \t")
 		    (or (= (following-char) ?#)
-			(let ((pps (parse-partial-sexp (point) end)))
-			  (or (nth 3 pps) (nth 4 pps) (nth 5 pps)))))))
+			;; Colon is special only after a label, or case ....
+			;; So quickly rule out most other uses of colon
+			;; and do no indentation for them.
+			(and (eq last-command-char ?:)
+			     (not (looking-at "case[ \t]"))
+			     (save-excursion
+			       (forward-word 1)
+			       (skip-chars-forward " \t")
+			       (< (point) end)))
+			(progn
+			  (beginning-of-defun)
+			  (let ((pps (parse-partial-sexp (point) end)))
+			    (or (nth 3 pps) (nth 4 pps) (nth 5 pps))))))))
 	(progn
 	  (insert last-command-char)
 	  (c++-indent-line)
 	  (and c-auto-newline
 	       (not (c-inside-parens-p))
 	       (progn
-		 (setq insertpos (1- (point)))
+		 ;; the new marker object, used to be just an integer
+		 (setq insertpos (make-marker))
+		 ;; changed setq to set-marker
+		 (set-marker insertpos (1- (point)))
+		 ;; do this before the newline, since in auto fill can break
 		 (newline)
-		 (c++-indent-line)))
+		 (c-indent-line)))
 	  (save-excursion
 	    (if insertpos (goto-char (1+ insertpos)))
 	    (delete-char -1))))
@@ -234,18 +307,7 @@ no args,if that value is non-nil."
 	  (self-insert-command (prefix-numeric-value arg)))
       (self-insert-command (prefix-numeric-value arg)))))
 
-(defun c-inside-parens-p ()
-  (condition-case ()
-      (save-excursion
-	(save-restriction
-	  (narrow-to-region (point)
-			    (progn (beginning-of-defun) (point)))
-	  (goto-char (point-max))
-	  (= (char-after (or (scan-lists (point) -1 1) (point-min))) ?\()))
-    (error nil)))
-
 (defun c++-indent-command (&optional whole-exp)
-  (interactive "P")
   "Indent current line as C++ code, or in some cases insert a tab character.
 If c-tab-always-indent is non-nil (the default), always indent current line.
 Otherwise, indent the current line only if point is at the left margin
@@ -255,6 +317,7 @@ A numeric argument, regardless of its value,
 means indent rigidly all the lines of the expression starting after point
 so that this line becomes properly indented.
 The relative indentation among the lines of the expression are preserved."
+  (interactive "P")
   (if whole-exp
       ;; If arg, always indent this line as C
       ;; and shift remaining lines of expression the same amount.
@@ -296,7 +359,7 @@ Return the amount the indentation changed by."
 	  (t
 	   (skip-chars-forward " \t")
 	   (if (listp indent) (setq indent (car indent)))
-	   (cond ((looking-at "\\(public\\|private\\|protected\\):")
+	   (cond ((looking-at "\\(default\\|public\\|private\\|protected\\):")
 		  (setq indent (+ indent c-label-offset)))
 		 ((or (looking-at "case\\b")
 		      (and (looking-at "[A-Za-z]")
@@ -304,7 +367,8 @@ Return the amount the indentation changed by."
 			     (forward-sexp 1)
 			     (looking-at ":[^:]"))))
 		  (setq indent (max 1 (+ indent c-label-offset))))
-		 ((looking-at "else\\b")
+		 ((and (looking-at "else\\b")
+		       (not (looking-at "else\\s_")))
 		  (setq indent (save-excursion
 				 (c-backward-to-start-of-if)
 				 (current-indentation))))
@@ -348,31 +412,59 @@ Returns nil if line starts inside a string, t if in a comment."
 	     ;; return nil or t if should not change this line
 	     (nth 4 state))
 	    ((null containing-sexp)
-	     ;; Line is at top level.  May be data or function definition,
-	     ;; or may be function argument declaration.
-	     ;; Indent like the previous top level line
-	     ;; unless that ends in a closeparen without semicolon,
-	     ;; in which case this line is the first argument decl.
+	     ;; Line is at top level.  May be data or function definition, or
+	     ;; may be function argument declaration or member initialization.
+	     ;; Indent like the previous top level line unless
+	     ;; (1) the previous line ends in a closeparen without semicolon,
+	     ;; in which case this line is the first argument declaration or
+	     ;; member initialization, or
+	     ;; (2) the previous line begins with a colon,
+	     ;; in which case this is the second line of member inits.
+	     ;; It is assumed that arg decls and member inits are not mixed.
 	     (goto-char indent-point)
 	     (skip-chars-forward " \t")
 	     (if (= (following-char) ?{)
 		 0   ; Unless it starts a function body
 	       (c++-backward-to-noncomment (or parse-start (point-min)))
 	       (if (= (preceding-char) ?\))
-		   c-argdecl-indent
+		   (progn		; first arg decl or member init
+		     (goto-char indent-point)
+		     (skip-chars-forward " \t")
+		     (if (= (following-char) ?:)
+			 c++-member-init-indent
+		       c-argdecl-indent))
 		 (if (= (preceding-char) ?\;)
 		     (backward-char 1))
 		 (if (= (preceding-char) ?})
 		     0
-		   (current-indentation))
+		   (beginning-of-line)	; continued arg decls or member inits
+		   (skip-chars-forward " \t")
+		   (if (= (following-char) ?:)
+		       (if c++-continued-member-init-offset
+			   (+ (current-indentation)
+			      c++-continued-member-init-offset)
+			 (progn
+			   (forward-char 1)
+			   (skip-chars-forward " \t")
+			   (current-column)))
+		     (current-indentation)))
 		 )))
 	    ((/= (char-after containing-sexp) ?{)
 	     ;; line is expression, not statement:
 	     ;; indent to just after the surrounding open -- unless
 	     ;; empty arg list, in which case we do what
 	     ;; c++-empty-arglist-indent says to do.
-	     (if (and (null (nth 2 state))	;; indicates empty arg list.
-		      c++-empty-arglist-indent)
+	     (if (and c++-empty-arglist-indent
+		      (or (null (nth 2 state))	;; indicates empty arg
+						;; list.
+			  ;; Use a heuristic: if the first
+			  ;; non-whitespace following left paren on
+			  ;; same line is not a comment,
+			  ;; is not an empty arglist.
+			  (save-excursion
+			    (goto-char (1+ containing-sexp))
+			    (not
+			     (looking-at "\\( \\|\t\\)*[^/\n]")))))
 		 (progn
 		   (goto-char containing-sexp)
 		   (beginning-of-line)
@@ -380,8 +472,12 @@ Returns nil if line starts inside a string, t if in a comment."
 		   (goto-char (min (+ (point) c++-empty-arglist-indent)
 				   (1+ containing-sexp)))
 		   (current-column))
-	     (goto-char (1+ containing-sexp))
-	     (current-column)))
+	       ;; In C-mode, we would always indent to one after the
+	       ;; left paren.  Here, though, we may have an
+	       ;; empty-arglist, so we'll indent to the min of that
+	       ;; and the beginning of the first argument.
+	       (goto-char (1+ containing-sexp))
+	       (current-column)))
 	    (t
 	     ;; Statement.  Find previous non-comment character.
 	     (goto-char indent-point)
@@ -417,10 +513,10 @@ Returns nil if line starts inside a string, t if in a comment."
 			    (forward-line 1))
 			   (t
 			    (re-search-forward ":[^:]" nil 'move))))
-		   ;; The first following code counts
-		   ;; if it is before the line we want to indent.
-		   (and (< (point) indent-point)
-			(current-column)))
+		      ;; The first following code counts
+		      ;; if it is before the line we want to indent.
+		      (and (< (point) indent-point)
+			   (current-column)))
 		 ;; If no previous statement,
 		 ;; indent it relative to line brace is on.
 		 ;; For open brace in column zero, don't let statement
@@ -431,28 +527,19 @@ Returns nil if line starts inside a string, t if in a comment."
 		 (+ (if (and (bolp) (zerop c-indent-level))
 			(+ c-brace-offset c-continued-statement-offset)
 		      c-indent-level)
-		    (if (save-excursion (skip-chars-backward " \t")
-					(bolp))
-			0 c-brace-imaginary-offset)
-		    (current-indentation)))))))))
-
-(defun calculate-c-indent-within-comment ()
-  "Return the indentation amount for line, assuming that
-the current line is to be regarded as part of a block comment."
-  (let (end star-start)
-    (save-excursion
-      (beginning-of-line)
-      (skip-chars-forward " \t")
-      (setq star-start (= (following-char) ?\*))
-      (skip-chars-backward " \t\n")
-      (setq end (point))
-      (beginning-of-line)
-      (skip-chars-forward " \t")
-      (and (re-search-forward "/\\*[ \t]*" end t)
-	   star-start
-	   (goto-char (1+ (match-beginning 0))))
-      (current-column))))
-
+		    ;; Move back over whitespace before the openbrace.
+		    ;; If openbrace is not first nonwhite thing on the line,
+		    ;; add the c-brace-imaginary-offset.
+		    (progn (skip-chars-backward " \t")
+			   (if (bolp) 0 c-brace-imaginary-offset))
+		    ;; If the openbrace is preceded by a parenthesized exp,
+		    ;; move to the beginning of that;
+		    ;; possibly a different line
+		    (progn
+		      (if (eq (preceding-char) ?\))
+			  (forward-sexp -1))
+		      ;; Get initial indentation of the line we are on.
+		      (current-indentation))))))))))
 
 (defun c++-backward-to-noncomment (lim)
   (let (opoint stop)
@@ -474,30 +561,6 @@ the current line is to be regarded as part of a block comment."
 	       (setq stop t)
 	       (goto-char opoint)))))))
 
-(defun c-backward-to-start-of-continued-exp (lim)
-  (if (= (preceding-char) ?\))
-      (forward-sexp -1))
-  (beginning-of-line)
-  (if (<= (point) lim)
-      (goto-char (1+ lim)))
-  (skip-chars-forward " \t"))
-
-(defun c-backward-to-start-of-if (&optional limit)
-  "Move to the start of the last ``unbalanced'' if."
-  (or limit (setq limit (save-excursion (beginning-of-defun) (point))))
-  (let ((if-level 1)
-	(case-fold-search nil))
-    (while (not (zerop if-level))
-      (backward-sexp 1)
-      (cond ((looking-at "else\\b")
-	     (setq if-level (1+ if-level)))
-	    ((looking-at "if\\b")
-	     (setq if-level (1- if-level)))
-	    ((< (point) limit)
-	     (setq if-level 0)
-	     (goto-char limit))))))
-
-
 (defun indent-c++-exp ()
   "Indent each line of the C++ grouping following point."
   (interactive)
@@ -506,7 +569,7 @@ the current line is to be regarded as part of a block comment."
 	(case-fold-search nil)
 	restart outer-loop-done inner-loop-done state ostate
 	this-indent last-sexp
-	at-else
+	at-else at-brace
 	(opoint (point))
 	(next-depth 0))
     (save-excursion
@@ -538,12 +601,16 @@ the current line is to be regarded as part of a block comment."
 	    (setq outer-loop-done t))
 	(if outer-loop-done
 	    nil
-	  (if (/= last-depth next-depth)
-	      (setq last-sexp nil))
-	  (while (> last-depth next-depth)
+	  ;; If this line had ..))) (((.. in it, pop out of the levels
+	  ;; that ended anywhere in this line, even if the final depth
+	  ;; doesn't indicate that they ended.
+	  (while (> last-depth (nth 6 state))
 	    (setq indent-stack (cdr indent-stack)
 		  contain-stack (cdr contain-stack)
 		  last-depth (1- last-depth)))
+	  (if (/= last-depth next-depth)
+	      (setq last-sexp nil))
+	  ;; Add levels for any parens that were started in this line.
 	  (while (< last-depth next-depth)
 	    (setq indent-stack (cons nil indent-stack)
 		  contain-stack (cons nil contain-stack)
@@ -567,17 +634,17 @@ the current line is to be regarded as part of a block comment."
 		  ;; Find last non-comment character before this line
 		  (save-excursion
 		    (setq at-else (looking-at "else\\W"))
+		    (setq at-brace (= (following-char) ?{))
 		    (c++-backward-to-noncomment opoint)
 		    (if (not (memq (preceding-char) '(nil ?\, ?\; ?} ?: ?{)))
 			;; Preceding line did not end in comma or semi;
 			;; indent this line  c-continued-statement-offset
 			;; more than previous.
 			(progn
-			  (c-backward-to-start-of-continued-exp
-			   (car contain-stack))
+			  (c-backward-to-start-of-continued-exp (car contain-stack))
 			  (setq this-indent
-				(+ c-continued-statement-offset
-				   (current-column))))
+				(+ c-continued-statement-offset (current-column)
+				   (if at-brace c-continued-brace-offset 0))))
 		      ;; Preceding line ended in comma or semi;
 		      ;; use the standard indent for this level.
 		      (if at-else
@@ -594,15 +661,14 @@ the current line is to be regarded as part of a block comment."
 	    ;; Adjust line indentation according to its contents
 	    (if (looking-at "\\(public\\|private\\|protected\\):")
 		(setq this-indent (- this-indent c-indent-level)))
-	    (if (and (looking-at "[A-Za-z]")
-		     (save-excursion
-		       (forward-sexp 1)
-		       (looking-at ":[^:]")))
+	    (if (or (looking-at "case[ \t]")
+		    (and (looking-at "[A-Za-z]")
+			 (save-excursion
+			   (forward-sexp 1)
+			   (looking-at ":[^:]"))))
 		(setq this-indent (max 1 (+ this-indent c-label-offset))))
-	    (if (looking-at "case[ \t]")
-		(setq this-indent (- this-indent c-indent-level)))
 	    (if (looking-at "friend[ \t]class[ \t]")
-		(setq this-indent (- this-indent c-indent-level)))
+		(setq this-indent (+ this-indent c++-friend-offset)))
 	    (if (= (following-char) ?})
 		(setq this-indent (- this-indent c-indent-level)))
 	    (if (= (following-char) ?{)
@@ -615,11 +681,9 @@ the current line is to be regarded as part of a block comment."
 		  (indent-to this-indent)))
 	    ;; Indent any comment following the text.
 	    (or (looking-at comment-start-skip)
-		(if (re-search-forward
-		     comment-start-skip
-		     (save-excursion (end-of-line) (point))
-		     t)
-		    (progn (indent-for-comment) (beginning-of-line))))))))))
+		(if (re-search-forward comment-start-skip (save-excursion (end-of-line) (point)) t)
+		    (progn (indent-for-comment) (beginning-of-line)))))))))
+  )
 
 (defun fill-C-comment ()
   (interactive)
@@ -647,15 +711,6 @@ line."
     (beginning-of-line)
     (point)))
 
-(defun previous-line-starts-with (string)
-  "Returns t if previous nonblank line begins with STRING"
-  (save-excursion
-    (beginning-of-line 0)
-    (skip-chars-forward " \t")
-    (if (eolp)
-	(previous-line-starts-with string)
-      (looking-at string))))
-
 (defun c++-insert-header ()
   "Insert header denoting C++ code at top of buffer."
   (interactive)
@@ -680,6 +735,7 @@ line."
       (setq pos (1+ pos)))
     count))
 
+
 
 ;;; This page covers "macroization;" making C++ parameterized types
 ;;; via macros.
@@ -723,6 +779,38 @@ so that indentation will work right."
 	(progn (skip-chars-backward " \t")
 	       (kill-line))))))
 
+
+;;; This page covers commenting out multiple lines.
+
+(defun c++-comment-region ()
+  "Comment out all lines in a region between mark and current point by
+inserting \"// \" (comment-start)in front of each line."
+  (interactive)
+  (let ((start (mark)))
+    (save-excursion
+      (while (>= (point) start)
+	(beginning-of-line)
+	(insert comment-start)
+	(previous-line 1)))))
+
+
+(defun c++-uncomment-region ()
+  "Uncomment all lines in region between mark and current point by deleting
+the leading \"// \" from each line, if any."
+  (interactive)
+  (let ((start (mark))
+	(n     (length comment-start))
+ 	(char  (strings-to-char comment-start)))
+    (save-excursion
+      (while (>= (point) start)
+ 	(beginning-of-line)
+ 	(if (looking-at (concat " *" comment-start))
+	    (progn
+	      (zap-to-char 1 char)
+	      (delete-char n)))
+ 	(previous-line 1)))))
+
+
 ;;; Below are two regular expressions that attempt to match defuns
 ;;; "strongly" and "weakly."  The strong one almost reconstructs the
 ;;; grammar of C++; the weak one just figures anything id or curly on
@@ -787,7 +875,7 @@ if nonNIL, use c++-defun-header-strong")
   "Strongly-defined regexp to match beginning of structure \
 or function definition.  ")
 
-
+
 ;; This part has to do with recognizing defuns.
 
 ;; The weak convention we will use is that a defun begins any time
