@@ -66,8 +66,8 @@ protected:
  public:
                     BitStrBit(BitString* v, int p);
                     BitStrBit(BitStrBit& b);
-                    ~BitStrBit();
-  int               operator int();
+                   ~BitStrBit();
+                    operator int();
   int               operator =  (int b);
   int               operator == (int b);
   int               operator != (int b);
@@ -725,7 +725,7 @@ inline BitStrBit::BitStrBit(BitString* v, int p)
 
 inline BitStrBit::~BitStrBit() {}
 
-inline int BitStrBit::operator int()
+inline BitStrBit::operator int()
 {
   return src->test(pos);
 }
