@@ -26,9 +26,9 @@ and this notice must be preserved on all copies.
 
 #include <BitString.h>
 #include <std.h>
-#include "libconfig.h"
+#include <values.h>
 #include <Obstack.h>
-
+#include <AllocQueue.h>
 
 void BitString::error(char* msg)
 {
@@ -340,14 +340,16 @@ BitStrRep* BStr_resize(BitStrRep* old, int newlen)
 
 BitStrRep* BStr_copy(BitStrRep* old, BitStrRep* src)
 {
-  BitStrRep* rep;
   if (old == src) return old; 
+  BitStrRep* rep;
   if (old == &_nilBitStrRep) old = 0;
   if (src == &_nilBitStrRep) src = 0;
   if (src == 0)
   {
     if (old == 0)
       rep = BSnew(0);
+    else
+      rep = old;
     rep->len = 0;
   }
   else 
@@ -974,7 +976,6 @@ int BitString::previous(int p, int b = 1)
   int pos = BitStr_pos(p);
 
   unsigned short* s = rep->s;
-  int l = BitStr_len(rep->len);
 
   if (p >= rep->len)
   {
@@ -1060,7 +1061,7 @@ int BitString::search(int startx, int lengthx,
     int ypos = BitStr_pos(starty);
     
     int rightxind = BitStr_index(rightx);
-    int leftxind = BitStr_index(leftx);
+
     unsigned short x = borrow_hi(xs, xind, rightxind, xpos);
   
     int rightyind = BitStr_index(righty);
@@ -1107,7 +1108,7 @@ int BitString::search(int startx, int lengthx,
   }
   else
   {
-    int leftx = startx;
+
     int rightx = lengthx - 1;
     if (ylen == 0) return startx;
     if (starty < 0 || righty < 0 || startx < 0 || startx >= lengthx) return -1;
@@ -1118,7 +1119,7 @@ int BitString::search(int startx, int lengthx,
     int ypos = BitStr_pos(starty);
 
     int rightxind = BitStr_index(rightx);
-    int leftxind = BitStr_index(leftx);
+
     unsigned short x = borrow_hi(xs, xind, rightxind, xpos);
     unsigned short nextx = (xind >= rightxind) ? 0 : (xs[xind+1] >> xpos);
   
@@ -1195,7 +1196,7 @@ int BitPattern::search(unsigned short* xs, int startx, int lengthx)
     int xpos = BitStr_pos(startx);
     
     int rightxind = BitStr_index(rightx);
-    int leftxind = BitStr_index(leftx);
+
     int rightmind = BitStr_index(rightm);
     int rightyind = BitStr_index(righty);
     
@@ -1233,7 +1234,7 @@ int BitPattern::search(unsigned short* xs, int startx, int lengthx)
   }
   else
   {
-    int leftx = startx;
+
     int rightx = lengthx - 1;
 
     if (righty < 0) return startx;
@@ -1243,7 +1244,7 @@ int BitPattern::search(unsigned short* xs, int startx, int lengthx)
     int xpos = BitStr_pos(startx);
     
     int rightxind = BitStr_index(rightx);
-    int leftxind = BitStr_index(leftx);
+
     int rightmind = BitStr_index(rightm);
     int rightyind = BitStr_index(righty);
     
@@ -1649,23 +1650,28 @@ BitStrTmp reverse(BitString& x)
 extern Obstack _libgxx_io_ob;
 extern char* _libgxx_io_oblast;
 
+extern AllocQueue _libgxx_fmtq;
 
 const char* BitStringtoa(BitString& x, char f = '0', char t = '1')
 {
-  if (_libgxx_io_oblast)  _libgxx_io_ob.free(_libgxx_io_oblast);
+  int wrksiz = x.length() + 2;
+  char* fmtbase = _libgxx_fmtq.alloc(wrksiz);
+  char* fmt = fmtbase;
   unsigned int  xl = x.rep->len;
   unsigned short* s = x.rep->s;
-  unsigned short a;
+  unsigned short a = 0;
 
   for (unsigned int  i = 0; i < xl; ++i)
   {
     if (i % BITSTRBITS == 0)
       a = *s++;
-    _libgxx_io_ob.grow((a & 1)? t : f);
+    *fmt++ = (a & 1)? t : f;
     a >>= 1;
   }
 
-  return _libgxx_io_oblast = (char*)(_libgxx_io_ob.finish(0));
+  *fmt = 0;
+
+  return fmtbase;
 }
 
 BitStrTmp atoBitString(const char* s, char f = '0', char t = '1')
@@ -1717,14 +1723,18 @@ ostream& operator << (ostream& s, BitString& x)
 
 const char* BitPatterntoa(BitPattern& p, char f = '0',char t = '1',char x='X')
 {
-  if (_libgxx_io_oblast)  _libgxx_io_ob.free(_libgxx_io_oblast);
   unsigned int  pl = p.pattern.rep->len;
   unsigned int  ml = p.mask.rep->len;
   unsigned int  l = pl <? ml;
+
+  int wrksiz = l + 2;
+  char* fmtbase = _libgxx_fmtq.alloc(wrksiz);
+  char* fmt = fmtbase;
+
   unsigned short* ps = p.pattern.rep->s;
   unsigned short* ms = p.mask.rep->s;
-  unsigned short a;
-  unsigned short m;
+  unsigned short a = 0;
+  unsigned short m = 0;
 
   for (unsigned int  i = 0; i < l; ++i)
   {
@@ -1734,14 +1744,15 @@ const char* BitPatterntoa(BitPattern& p, char f = '0',char t = '1',char x='X')
       m = *ms++;
     }
     if (m & 1)
-      _libgxx_io_ob.grow((a & 1)? t : f);
+      *fmt++ =(a & 1)? t : f;
     else
-      _libgxx_io_ob.grow(x);
+      *fmt++ = x;
     a >>= 1;
     m >>= 1;
   }
 
-  return _libgxx_io_oblast = (char*)(_libgxx_io_ob.finish(0));
+  *fmt = 0;
+  return fmtbase;
 }
 
 BitPattern atoBitPattern(const char* s,char f = '0',char t = '1',char x = 'X')
